**Role Enum**

This project provides utilities to formalize roles as enums.

The end-goal of this project is to be able to derive real role [configuration]
from enums, removing the need for configuration YAMLs.

The project effectively allows referencing roles with the enum:

```php
\Drupal\my_project\Enum\MyRoles::ContentEditor
```

And modify roles on a user with:

```php
$user->r->add(MyRoles::ContentEditor);
```

# Bundle class

Implement a bundle class for the User entity. I'd recommend using [BCA](https://www.drupal.org/project/bca)
to ease the process.

```php
namespace Drupal\my_project\Entity;

#[Bundle(
  entityType: 'user',
  bundle: 'user',
)]
final class MyProjectUser extends \Drupal\user\Entity\User implements \Drupal\role_enum\Entity\RoleInterface {

  use \Drupal\role_enum\Entity\Trait\RoleTrait;

}
```

The class should extend the User entity class, and implement interface `\Drupal\role_enum\Entity\RoleInterface`

The class should use the provided trait.

Note: methods may be introduced to the trait or interface, so it's best to use
the ones provided by the project in order to comply. Though certainly not a
requirement.

# Role enums

A role enum looks like:

```php
<?php
namespace Drupal\my_project\Enum;

enum TestRoles: string implements \Drupal\role_enum\Enum\ConfigBackedRoleInterface {

  case ContentEditor = 'editor';
  case Developer = 'admin';
  case User_Manager = 'user_manager';

  public function id(): string {
    return $this->value;
  }

  public function label(): string {
    return $this->name;
  }

}
```

The enum simply needs to implement the interface, it does not to be string backed.

Preferably the file is placed in `src/Enum/` of a module, if you want to change
this you'll need to customize the `role_enum.namespaces` container parameter.

# Optional enhancement

A helper is available to make the methods provided by the trait and interface less verbose.

Sadly, finding a good name for methods is difficult, so something familiar to
_field API-style_ was chosen:

```php
$user->r->add(MyRoles::ContentEditor);
$user->r->remove(MyRoles::ContentEditor);
$user->r->has(MyRoles::ContentEditor);
```

Override the entity constructor with the following.

```php
final class MyProjectUser {
  public \Drupal\role_enum\RoleEnumProxy $r;

  public function __construct(array $values, $entity_type, $bundle = FALSE, $translations = []) {
    parent::__construct($values, $entity_type, $bundle, $translations);
    $this->r = \Drupal\role_enum\RoleEnumProp::createProperty($this);
  }
}
```

Since the property is not part of any interface, or required by this module, the
property can be name anything. Except an in-use field names, like `roles`.

# Test utilities

If your project uses [Drupal Test Traits (DTT)](https://gitlab.com/weitzman/drupal-test-traits) to test a working version of your project,
you can bring traits into your project to ensure role enums stay in sync.

Create a new test, and use the trait.

```php
namespace Drupal\Tests\my_project\Functional;

final class MyProjectUserRoleEnumTest extends \weitzman\DrupalTestTraits\ExistingSiteBase {

  use \Drupal\role_enum\Dtt\RoleEnumTestTrait;

}
```
